//
//  ATContentController.m
//  MyAnimatedTableView
//
//  Created by countryhu on 12-12-31.
//  Copyright (c) 2012年 Thunder. All rights reserved.
//

#import "ATContentController.h"
#import "ATDesktopEntity.h"
#import "ATImageTextCell.h"
#import "ATColorView.h"

@implementation ATContentController

-(void)awakeFromNib{
    NSLog(@"awakeFromNib");
    _tableContents = [NSMutableArray array];
    
    NSURL *url = [NSURL fileURLWithPath:@"/Library/Desktop Pictures/" isDirectory:YES];
    ATDesktopFolderEntity *desktopFolder = [[ATDesktopFolderEntity alloc] initWithFileUrl:url];
    [_tableContents addObject:desktopFolder];
    for (ATDesktopEntity *entity in [desktopFolder children]) {
        if ([entity isKindOfClass:[ATDesktopImageEntity class]]) {
            [_tableContents addObject:entity];
        }
    }
    // subImages on level
    for (ATDesktopEntity *entity in [desktopFolder children]) {
        if ([entity isKindOfClass:[ATDesktopFolderEntity class]]) {
            ATDesktopFolderEntity *subFolderEntity = (ATDesktopFolderEntity *)entity;
            for (ATDesktopEntity *subEntity in [subFolderEntity children]) {
                [_tableContents addObject:subEntity];
            }
        }
    }
    
    NSImage *initialImage = [[NSImage alloc] initByReferencingURL:[[NSWorkspace sharedWorkspace] desktopImageURLForScreen:[NSScreen mainScreen]]];
    [_imageViewMain setImage:initialImage];
}

#pragma mark -
#pragma mark NSTableView delegate and dataSource
-(NSInteger)numberOfRowsInTableView:(NSTableView *)aTableView{
    return [_tableContents count];
}

-(id)tableView:(NSTableView *)aTableView objectValueForTableColumn:(NSTableColumn *)aTableColumn row:(NSInteger)rowIndex{
    return [[_tableContents objectAtIndex:rowIndex] title];
}

- (NSCell *)tableView:(NSTableView *)tableView dataCellForTableColumn:(NSTableColumn *)tableColumn row:(NSInteger)row {
    ATDesktopImageEntity *imageEntity = [self _imageEntityForRow:row];
    if (imageEntity != nil) {
        return [tableColumn dataCell];
    } else {
        return _sharedGroupTitleCell;
    }
}

- (void)tableView:(NSTableView *)aTableView willDisplayCell:(id)aCell forTableColumn:(NSTableColumn *)aTableColumn row:(NSInteger)rowIndex {
    ATDesktopImageEntity *imageEntity = [self _imageEntityForRow:rowIndex];
    if (imageEntity != nil) {
        ATImageTextCell *imageTextCell = (ATImageTextCell *)aCell;
        imageTextCell.image = imageEntity.thumbnailImage;
        imageTextCell.fillColor = imageEntity.fillColor;
        imageTextCell.fillColorName = imageEntity.fillColorName;
    }
}

- (CGFloat)tableView:(NSTableView *)tableView heightOfRow:(NSInteger)row {
    if ([[_tableContents objectAtIndex:row] isKindOfClass:[ATDesktopFolderEntity class]]) {
        return [tableView rowHeight];
    } else {
        return 75.0;
    }
}

- (BOOL)tableView:(NSTableView *)tableView shouldTrackCell:(NSCell *)cell forTableColumn:(NSTableColumn *)tableColumn row:(NSInteger)row {
    NSInteger column = tableColumn ? [[tableView tableColumns] indexOfObject:tableColumn] : -1;
    NSUInteger hitTest = [cell hitTestForEvent:[NSApp currentEvent] inRect:[tableView frameOfCellAtColumn:column row:row] ofView:tableView];
    return (hitTest & NSCellHitTrackableArea) != 0;
}


- (BOOL)tableView:(NSTableView *)tableView isGroupRow:(NSInteger)row {
    if ([[_tableContents objectAtIndex:row] isKindOfClass:[ATDesktopFolderEntity class]]) {
        return YES;
    } else {
        return NO;
    }
}

- (NSIndexSet *)tableView:(NSTableView *)tableView selectionIndexesForProposedSelection:(NSIndexSet *)proposedSelectionIndexes {
    NSInteger row = [tableView clickedRow];
    if (row != -1 && ![self tableView:tableView isGroupRow:row]) {
        ATImageTextCell *cell = (ATImageTextCell *)[tableView preparedCellAtColumn:0 row:row];
        NSUInteger hitTest = [cell hitTestForEvent:[NSApp currentEvent] inRect:[tableView frameOfCellAtColumn:0 row:row] ofView:tableView];
        if ((hitTest & NSCellHitTrackableArea) != 0) {
            return [tableView selectedRowIndexes];
        }
    } else {
        if (proposedSelectionIndexes.count == 0 || [self tableView:tableView isGroupRow:row]) {
            return [tableView selectedRowIndexes];
        }
    }
    
    return proposedSelectionIndexes;
}

- (void)tableViewSelectionDidChange:(NSNotification *)aNotification {
    NSInteger selectedRow = [_tableViewMain selectedRow];
    ATDesktopImageEntity *entityImage = [self _imageEntityForRow:selectedRow];
    if (entityImage != nil) {
        [_imageViewMain setImage:entityImage.image];
        [[_colorViewMain animator] setBackgroundColor:entityImage.fillColor];
        [_imageViewMain.window displayIfNeeded];
    }
}


//
//- (BOOL)tableView:(NSTableView *)tableView shouldTrackCell:(NSCell *)cell forTableColumn:(NSTableColumn *)tableColumn row:(NSInteger)row {
//    NSInteger column = tableColumn ? [[tableView tableColumns] indexOfObject:tableColumn] : -1;
//    NSUInteger hitTest = [cell hitTestForEvent:[NSApp currentEvent] inRect:[tableView frameOfCellAtColumn:column row:row] ofView:tableView];
//    return (hitTest & NSCellHitTrackableArea) != 0;
//}

 - (ATDesktopImageEntity *)_imageEntityForRow:(NSInteger)row {
     if (row != -1) {
         id result = _tableContents[row];
         if ([result isKindOfClass:[ATDesktopImageEntity class]]) {
             return result;
         }
     }
     return nil;
 }

#pragma mark -
#pragma mark NSSplitViewDelegate methods
- (CGFloat)splitView:(NSSplitView *)splitView constrainMaxCoordinate:(CGFloat)proposedMax ofSubviewAt:(NSInteger)dividerIndex {
    CGFloat splitWidth = splitView.bounds.size.width;
    return splitWidth - 200;
}

- (CGFloat)splitView:(NSSplitView *)splitView constrainMinCoordinate:(CGFloat)proposedMin ofSubviewAt:(NSInteger)dividerIndex {
    return 200;
}

@end
