//
//  ATDesktopEntity.m
//  MyAnimatedTableView
//
//  Created by countryhu on 12-12-31.
//  Copyright (c) 2012年 Thunder. All rights reserved.
//

#import "ATDesktopEntity.h"

#define THUMBNAIL_HEIGHT 71.0

@implementation ATDesktopEntity

@synthesize fileUrl = _fileUrl;

-(id)initWithFileUrl:(NSURL *)fileUrl {
    self = [super init];
    _fileUrl = fileUrl;
    return self;
}

-(NSString *)title {
    NSString *result;
    if ([_fileUrl getResourceValue:&result forKey:NSURLLocalizedNameKey error:NULL]) {
        return result;
    }
    return nil;
}

@end


@implementation ATDesktopFolderEntity

-(NSArray *)children{
    if (_children) {
        return _children;
    }
    NSArray *imageUTIs = [NSImage imageTypes];
    NSError *error;
    // wfq::todo process in assistant thread
    NSArray *urls = [[NSFileManager defaultManager] contentsOfDirectoryAtURL:self.fileUrl includingPropertiesForKeys:[NSArray arrayWithObject:NSURLLocalizedNameKey] options:NSDirectoryEnumerationSkipsHiddenFiles|NSDirectoryEnumerationSkipsSubdirectoryDescendants error:&error];
    NSMutableArray *newChildren = [[NSMutableArray alloc] initWithCapacity:[urls count]];
    for (NSURL *url in urls) {
        NSString *uti;
        if ([url getResourceValue:&uti forKey:NSURLTypeIdentifierKey error:NULL]) {
            ATDesktopEntity *entity;
            if ([imageUTIs containsObject:uti]) {
                entity = [[ATDesktopImageEntity alloc] initWithFileUrl:url];
                [newChildren addObject:entity];
            } else if ([uti isEqualToString:(NSString *)kUTTypeFolder]) {
                entity = [[ATDesktopFolderEntity alloc] initWithFileUrl:url];
                [newChildren addObject:entity];
            }
        }
    }
    _children = newChildren;
    return _children;
}

@end

@implementation ATDesktopImageEntity

- (NSImage *)_getThumbnailImage:(NSImage *)srcImage {
    NSSize srcImageSize = [srcImage size];
    CGFloat srcImageRatio = srcImageSize.width / srcImageSize.height;
    NSSize thumbnailSize = NSMakeSize(THUMBNAIL_HEIGHT*srcImageRatio, THUMBNAIL_HEIGHT);
    NSImage *thumbnailImage = [[NSImage alloc] initWithSize:thumbnailSize];
    [thumbnailImage lockFocus];
    // wfq::todo slow operation
    [srcImage drawInRect:NSMakeRect(0, 0, thumbnailSize.width, thumbnailSize.height) fromRect:NSZeroRect operation:NSCompositeSourceOver fraction:1.0];
    [thumbnailImage unlockFocus];
    return thumbnailImage;
}

- (id)initWithFileUrl:(NSURL *)fileUrl {
    self = [super initWithFileUrl:fileUrl];
    
    NSColorList *colorList = [NSColorList colorListNamed:@"Crayons"];
    NSArray *colorNameArray = [colorList allKeys];
    static NSInteger lastColorIndex = 0;
    if (lastColorIndex >= colorNameArray.count) {
        lastColorIndex = 0;
    }
    _fillColorName = [colorNameArray objectAtIndex:lastColorIndex];
    lastColorIndex ++;
    _fillColor = [colorList colorWithKey:_fillColorName];
    
    _image = [[NSImage alloc] initWithContentsOfURL:self.fileUrl]; // wfq::todo slow operation
    _thumbnailImage = [self _getThumbnailImage:_image]; // wfq::todo slow operation
    return self;
}

@synthesize image = _image;
@synthesize thumbnailImage = _thumbnailImage;
@synthesize fillColor = _fillColor;
@synthesize fillColorName = _fillColorName;

@end
