//
//  ATImageTextCell.m
//  MyAnimatedTableView
//
//  Created by countryhu on 13-1-7.
//  Copyright (c) 2013年 Thunder. All rights reserved.
//

#import "ATImageTextCell.h"
#import "ATColorCell.h"

#define IMAGE_INSET 8.0
#define ASPECT_RATIO 1.6
#define INSET_FROM_IMAGE_TO_TEXT 4.0
#define FILL_COLOR_RECT_HEIGHT 25.0

@implementation ATImageTextCell

// wfq::todo
//- (id)init {
//    self = [super init];
//    _fillColorCell = [[ATColorCell alloc] init];
//    return self;
//}

- (id)copyWithZone:(NSZone *)zone {
    ATImageTextCell *result = [super copyWithZone:zone];
    if (result != nil) {
        result->_imageCell = [_imageCell copyWithZone:zone];
    }
    return result;
}

@dynamic image;
- (NSImage*)image:(NSImage *)image {
    return _imageCell.image;
}

- (void)setImage:(NSImage *)image {
    if (_imageCell == nil) {
        _imageCell = [[NSImageCell alloc] init];
        [_imageCell setImageScaling:NSImageScaleNone];
    }
    [_imageCell setImage:image];
}

- (void)_ensureFillColorCellCreated {
    if (_fillColorCell == nil) {
        _fillColorCell = [[ATColorCell alloc] init];
        [_fillColorCell setTextColor:[NSColor grayColor]];
    }
}
@dynamic fillColor;
- (NSColor *)fillColor {
    return _fillColorCell.color;
}

- (void)setFillColor:(NSColor *)fillColor {
    [self _ensureFillColorCellCreated];
    _fillColorCell.color = fillColor;
}

@dynamic fillColorName;
- (NSString *)fillColorName {
    return _fillColorCell.title;
}

- (void)setFillColorName:(NSString *)fillColorName {
    [self _ensureFillColorCellCreated];
    _fillColorCell.title = fillColorName;
}

- (NSRect)_imageFrameForInteriorFrame:(NSRect)interiorFrame {
    NSRect imageFrame = interiorFrame;
    imageFrame.origin.x += IMAGE_INSET;
    imageFrame.origin.y += IMAGE_INSET;
    imageFrame.size.height -= 2*IMAGE_INSET;
    imageFrame.size.width = imageFrame.size.height*ASPECT_RATIO;
    return imageFrame;
}

- (NSRect)imageRectForBounds:(NSRect)frame {
    // We would apply any inset that here that drawWithFrame did before calling
    // drawInteriorWithFrame:. It does none, so we don't do anything.
    return [self _imageFrameForInteriorFrame:frame];
}
- (NSRect)_titleFrameForInteriorFrame:(NSRect)interiorFrame {
    NSRect imageFrame = [self _imageFrameForInteriorFrame:interiorFrame];
    NSRect result = interiorFrame;
    result.origin.x = NSMaxX(imageFrame) + INSET_FROM_IMAGE_TO_TEXT;
    result.size.width = NSMaxX(interiorFrame) - NSMinX(result);
    NSSize naturalSize = [super cellSize];
    result.origin.y = floor(NSMidY(imageFrame) - naturalSize.height - INSET_FROM_IMAGE_TO_TEXT);
    result.size.height = naturalSize.height;
    return result;
}

- (NSRect)_fillColorFrameForInteriorFrame:(NSRect)interiorFrame {
    NSRect imageFrame = [self _imageFrameForInteriorFrame:interiorFrame];
    NSRect result = interiorFrame;
    result.origin.x = NSMaxX(imageFrame) + INSET_FROM_IMAGE_TO_TEXT;
    result.size.width = NSMaxX(interiorFrame) - NSMinX(result);
    result.origin.y = NSMidY(imageFrame);
    result.size.height = FILL_COLOR_RECT_HEIGHT;
    return  result;
}

- (void)drawInteriorWithFrame:(NSRect)cellFrame inView:(NSView *)controlView {
    if (_imageCell != nil) {
        NSRect imageFrame = [self _imageFrameForInteriorFrame:cellFrame];
        [_imageCell drawWithFrame:imageFrame inView:controlView];
    }
    if (_fillColorCell) {
        NSRect fillColorFrame = [self _fillColorFrameForInteriorFrame:cellFrame];
        [_fillColorCell drawInteriorWithFrame:fillColorFrame inView:controlView];
    }
    
    NSRect titleFrame = [self _titleFrameForInteriorFrame:cellFrame];
    [super drawInteriorWithFrame:titleFrame inView:controlView];
}

//- (void)editWithFrame:(NSRect)aRect inView:(NSView *)controlView editor:(NSText *)textObj delegate:(id)anObject event:(NSEvent *)theEvent {
//    aRect = [self _titleFrameForInteriorFrame:aRect];
//    [super editWithFrame:aRect inView:controlView editor:textObj delegate:anObject event:theEvent];
//}

- (void)selectWithFrame:(NSRect)aRect inView:(NSView *)controlView editor:(NSText *)textObj delegate:(id)anObject start:(NSInteger)selStart length:(NSInteger)selLength {
    aRect = [self _titleFrameForInteriorFrame:aRect];
    [super selectWithFrame:aRect inView:controlView editor:textObj delegate:anObject start:selStart length:selLength];
}

- (NSUInteger)hitTestForEvent:(NSEvent *)event inRect:(NSRect)cellFrame ofView:(NSView *)controlView {
    NSPoint point = [controlView convertPoint:[event locationInWindow] fromView:nil];
    
    NSRect imageFrame = [self _imageFrameForInteriorFrame:cellFrame];
    if (NSPointInRect(point, imageFrame)) {
        return [_imageCell hitTestForEvent:event inRect:imageFrame ofView:controlView];
    }
    
    NSRect fillColorFrame = [self _fillColorFrameForInteriorFrame:cellFrame];
    if (NSPointInRect(point, fillColorFrame)) {
        return [_fillColorCell hitTestForEvent:event inRect:fillColorFrame ofView:controlView];
    }
    
    NSRect titleFrame = [self _titleFrameForInteriorFrame:cellFrame];
    if (NSPointInRect(point, titleFrame)) {
        return [super hitTestForEvent:event inRect:titleFrame ofView:controlView];
    }
    return NSCellHitNone;
}

- (BOOL)trackMouse:(NSEvent *)theEvent inRect:(NSRect)cellFrame ofView:(NSView *)controlView untilMouseUp:(BOOL)flag {
    NSRect fillColorFrame = [self _fillColorFrameForInteriorFrame:cellFrame];
    NSRect colorRectFrame = [_fillColorCell colorRectForFrame:fillColorFrame];
    NSPoint point = [controlView convertPoint:[theEvent locationInWindow] fromView:nil];
    if (NSPointInRect(point, colorRectFrame)) {
        NSLog(@"trackMouse in colorRectFrame!");
    }
}

@end
